package models

import (
	"fmt"

	"code.gitea.io/gitea/modules/log"
	"code.gitea.io/gitea/modules/timeutil"
)

type AiModelManage struct {
	ID            string             `xorm:"pk"`
	Name          string             `xorm:"NOT NULL"`
	Version       string             `xorm:"NOT NULL"`
	Parent        string             `xorm:"NOT NULL"`
	Type          int                `xorm:"NOT NULL"`
	Size          int64              `xorm:"NOT NULL"`
	Description   string             `xorm:"varchar(2000)"`
	Label         string             `xorm:"varchar(1000)"`
	Path          string             `xorm:"varchar(400) NOT NULL"`
	ConfigJson    string             `xorm:"text"`
	DownloadCount int                `xorm:"NOT NULL DEFAULT 0"`
	Engine        int                `xorm:"NOT NULL DEFAULT 0"`
	Status        int                `xorm:"NOT NULL DEFAULT 0"`
	Accuracy      string             `xorm:"varchar(1000)"`
	AttachmentId  string             `xorm:"NULL"`
	RepoId        int64              `xorm:"NULL"`
	CodeBranch    string             `xorm:"varchar(400) NULL"`
	CodeCommitID  string             `xorm:"NULL"`
	UserId        int64              `xorm:"NOT NULL"`
	TrainTaskInfo string             `xorm:"text NULL"`
	CreatedUnix   timeutil.TimeStamp `xorm:"created"`
	UpdatedUnix   timeutil.TimeStamp `xorm:"INDEX updated"`
}

func SaveModelToDb(model *AiModelManage) error {
	sess := x.NewSession()
	defer sess.Close()

	re, err := sess.Insert(model)
	if err != nil {
		return err
	}
	log.Info("success to save db.re=" + fmt.Sprint((re)))
	return nil
}

func DeleteModelById(id string) error {
	sess := x.NewSession()
	defer sess.Close()

	re, err := sess.Delete(&AiModelManage{
		ID: id,
	})
	if err != nil {
		return err
	}
	log.Info("success to delete from  db.re=" + fmt.Sprint((re)))
	return nil

}

func QueryModelByName(name string, uid int64) []*AiModelManage {
	sess := x.NewSession()
	defer sess.Close()
	sess.Select("*").Table("ai_model_manage").
		Where("name=" + name + " and user_id=" + fmt.Sprint(uid))
	aiModelManageList := make([]*AiModelManage, 0)
	sess.Find(&aiModelManageList)
	return aiModelManageList
}
